@@ -9,6 +9,7 @@ from itertools import izip
 from .functions        import parse_indices
 from .coordinatebounds import CoordinateBounds
 from .timeduration     import TimeDuration
+from .units            import Units
 from .variable         import Variable, SubspaceVariable
 
 from .data.data import Data
@@ -2788,87 +2789,99 @@ create bounds based on the coordinate array values.
 #            if not isinstance(cellsize, TimeDuration):
             cellsize = Data.asdata(abs(cellsize))
             if cellsize.Units:
-                if not cellsize.Units.equivalent(self.Units):
-                    raise ValueError("jhgsjhbd jh ")
-                cellsize.Units = self.Units
+                if self.Units.isreftime:
+                    if not cellsize.Units.istime:
+                        raise ValueError("q123423423jhgsjhbd jh ")
+                    cellsize.Units = Units(self.Units._utime.units)
+                else:
+                    if not cellsize.Units.equivalent(self.Units):
+                        raise ValueError("jhgsjhbd jh ")
+                    cellsize.Units = self.Units
             cellsize = cellsize.datum()
-        #--- End: def
+            
+            cellsize  = cellsize * 0.5
+            if not self.direction():
+                cellsize *= -1
+            
+            bounds = numpy_empty((size, 2), dtype=array.dtype)
+            bounds[:,0] = array - cellsize
+            bounds[:,1] = array + cellsize
 
 #        if isinstance(cellsize, TimeDuration):
 #            pass
 #            bounds = numpy_empty((size, 2), dtype=dtype)
 #            cellsize = None
-     
-        if bound is None:
-            # Creat Voronoi bounds
-            if size < 2:
-                if cellsize is not None:
-                    raise ValueError(
-"Can't create bounds for Voronoi cells from one value when cellsize parameter is unset")
+        else:
+            if bound is None:
+                # Creat Voronoi bounds
+                if size < 2:
+                    if cellsize is not None:
+                        raise ValueError(
+"Can't create bounds for Voronoi cells from one value")
 
-                cellsize  = cellsize * 0.5
-                if not self.direction():
-                    cellsize *= -1
-   
-                coord = array.item(0,)
-                bounds_1d = [coord - cellsize, coord + cellsize]
-            else:
+#    
+#                    cellsize  = cellsize * 0.5
+#                    if not self.direction():
+#                        cellsize *= -1
+#       
+#                    coord = array.item(0,)
+#                    bounds_1d = [coord - cellsize, coord + cellsize]
+#                else:
                 bounds_1d = [array.item(0,)*1.5 - array.item(1,)*0.5]
                 bounds_1d.extend((array[0:-1] + array[1:])*0.5)
                 bounds_1d.append(array.item(-1,)*1.5 - array.item(-2,)*0.5)
-
-            dtype = type(bounds_1d[0])
-
-        else:      
-            direction = self.direction()
-            if not direction and size > 1:
-                array = array[::-1]
-
-            bounds_1d = [bound]
-            if bound <= array.item(0,):
-                for i in xrange(size):
-                    bound = 2.0*array.item(i,) - bound
-                    bounds_1d.append(bound)
-            elif bound >= array.item(-1,):
-                for i in xrange(size-1, -1, -1):
-                    bound = 2.0*array.item(i,) - bound
-                    bounds_1d.append(bound)
-
-                bounds_1d = bounds_1d[::-1]
+    
+                dtype = type(bounds_1d[0])
+    
             else:
-                raise ValueError("bad bound value")
-
-            dtype = type(bounds_1d[-1])
-
-            if not direction:               
-                bounds_1d = bounds_1d[::-1]
-        #--- End: if
-
-        bounds = numpy_empty((size, 2), dtype=dtype)
-        bounds[:,0] = bounds_1d[:-1]
-        bounds[:,1] = bounds_1d[1:]        
-
-        if cellsize is not None:
-            cells = abs(bounds[:, 1] - bounds[:, 0])
-            cells = cells.squeeze(axis=1)
-            for i, (c, s) in enumerate(zip(array, cells)):
-                if s <= cellsize:
-                    continue
-                if self.direction():
-                    if c - bounds[i,0] <= cellsize:
-                        bounds[i,1] = bounds[i,0] + cellsize
-                    elif bounds[i,1] - c <= cellsize:
-                        bounds[i,0] = bounds[i,1] - cellsize
-                    else:
-                        raise ValueError("asdasdasds8888888888888888888888")
+                direction = self.direction()
+                if not direction and size > 1:
+                    array = array[::-1]
+    
+                bounds_1d = [bound]
+                if bound <= array.item(0,):
+                    for i in xrange(size):
+                        bound = 2.0*array.item(i,) - bound
+                        bounds_1d.append(bound)
+                elif bound >= array.item(-1,):
+                    for i in xrange(size-1, -1, -1):
+                        bound = 2.0*array.item(i,) - bound
+                        bounds_1d.append(bound)
+    
+                    bounds_1d = bounds_1d[::-1]
                 else:
-                    if bounds[i,0] - c <= cellsize:
-                        bounds[i,1] = bounds[i,0] - cellsize
-                    elif c - bounds[i,1] <= cellsize:
-                        bounds[i,0] = bounds[i,1] + cellsize
-                    else:
-                        raise ValueError("asdasdasds8888888888888888888888 66767")
-        #--- End: if
+                    raise ValueError("bad bound value")
+    
+                dtype = type(bounds_1d[-1])
+    
+                if not direction:               
+                    bounds_1d = bounds_1d[::-1]
+            #--- End: if
+
+            bounds = numpy_empty((size, 2), dtype=dtype)
+            bounds[:,0] = bounds_1d[:-1]
+            bounds[:,1] = bounds_1d[1:]        
+         #--- End: if
+#        if cellsize is not None:
+#            cells = abs(bounds[:, 1] - bounds[:, 0])
+#            for i, (c, s) in enumerate(zip(array, cells)):
+#                if s <= cellsize:
+#                    continue
+#                if self.direction():
+#                    if c - bounds[i,0] <= cellsize:
+#                        bounds[i,1] = bounds[i,0] + cellsize
+#                    elif bounds[i,1] - c <= cellsize:
+#                        bounds[i,0] = bounds[i,1] - cellsize
+#                    else:
+#                        raise ValueError("asdasdasds8888888888888888888888")
+#                else:
+#                    if bounds[i,0] - c <= cellsize:
+#                        bounds[i,1] = bounds[i,0] - cellsize
+#                    elif c - bounds[i,1] <= cellsize:
+#                        bounds[i,0] = bounds[i,1] + cellsize
+#                    else:
+#                        raise ValueError("asdasdasds8888888888888888888888 66767")
+#        #--- End: if
 
         bounds = CoordinateBounds(data=Data(bounds, self.Units), copy=False)
                            
@@ -2934,13 +2947,13 @@ None
                 value = value.override_units(self.Units)
             elif not units.equivalent(self.Units):
                 raise ValueError(
-"Period units {!r} are not equivalent to coordinate units {!r}".format(
+"Period units {0!r} are not equivalent to coordinate units {1!r}".format(
     units, self.Units))
 
             range = self.Data.range()
             if range >= value:
                 raise ValueError(
-"The coordinate range {!r} is not less than the period {!r}".format(
+"The coordinate range {0!r} is not less than the period {1!r}".format(
     range, value))
         #--- End: if
 
