@@ -118,6 +118,11 @@ def pmax(x, y):
             x = numpy_array(x)
     else:
         # x and y are both unmasked
+        if not x.ndim:
+            # Make sure that we have a numpy array (as opposed to,
+            # e.g. a numpy.float64)
+            x = numpy_asanyarray(x)
+
         numpy_maximum(x, y, out=x)
 
     return x
@@ -157,10 +162,15 @@ def pmin(x, y):
         # Only y is masked  
         z = numpy_minimum(x, y)
         x = numpy_ma_where(y.mask, x, z)
-        if x.mask is numpy_ma_nomask: #not x.mask.any():
+        if x.mask is numpy_ma_nomask:
             x = numpy_array(x)
     else:
-        # x and y are both unmasked
+        # x and y are both unmasked 
+        if not x.ndim:
+            # Make sure that we have a numpy array (as opposed to,
+            # e.g. a numpy.float64)
+            x = numpy_asanyarray(x)
+        
         numpy_minimum(x, y, out=x)
 
     return x
