@@ -36,9 +36,10 @@ from .functions import _close_netcdf_file, _open_netcdf_file
 
 def write(fields, filename, fmt='NETCDF3_CLASSIC', overwrite=True,
           verbose=False, cfa_options=None, mode='w',
-          least_significant_digit=None, endian='native',
-          compress=0, fletcher32=False, no_shuffle=False,
-          datatype=None, single=False, double=False):
+          least_significant_digit=None, endian='native', compress=0,
+          fletcher32=False, no_shuffle=False, datatype=None,
+          single=False, double=False, reference_datetime=None,
+          variable_attributes=None):
     '''Write fields to a CF-netCDF or CFA-netCDF file.
     
 NetCDF dimension and variable names will be taken from variables'
@@ -217,6 +218,18 @@ and auxiliary coordinate roles for different data variables.
     if least_significant_digit and fmt in ('CFA3', 'CFA4'):
         raise ValueError("Can't truncate data variables in {0} format file".format(fmt))
 
+
+    # ----------------------------------------------------------------
+    # Set up non-global attributes
+    # ----------------------------------------------------------------
+    if variable_attributes:
+        if isinstance(variable_attributes, basestring):
+            variable_attributes = set((variable_attributes,))
+        else:
+            variable_attributes = set(variable_attributes)
+    else:
+        variable_attributes = set()
+        
     # ----------------------------------------------------------------
     # Set up data type conversions
     # ----------------------------------------------------------------
@@ -262,12 +275,15 @@ and auxiliary coordinate roles for different data variables.
          'ncvar_names'      : set(()), # - Set of all netCDF
                                        #   dimension and netCDF
                                        #   variable names.
-         'global_properties': set(()), # - Set of global or
-                                       #   non-standard CF properties
-                                       #   which have identical
-                                       #   values across all input
-                                       #   fields.
-                                       #-----------------------------
+
+         'global_properties'  : set(()), # - Set of global or
+                                         #   non-standard CF properties
+                                         #   which have identical
+                                         #   values across all input
+                                         #   fields.
+                                         #-----------------------------
+         'variable_attributes': variable_attributes,
+                                         #-----------------------------
          'dimN'             : 1,       # - Counter
          'auxN'             : 1,       # - Counter
          'scalarN'          : 1,       # - Counter
@@ -310,7 +326,11 @@ and auxiliary coordinate roles for different data variables.
          # Specify data type conversions to be applied prior to writing
          # ------------------------------------------------------------
          'datatype'               : datatype,
-         }
+         # ------------------------------------------------------------
+         # Specify unit conversions to be applied prior to writing
+         # ------------------------------------------------------------
+         'reference_datetime'     : reference_datetime,
+    }
 
     if fmt == 'CFA3':
         g['cfa'] = True
@@ -322,7 +342,6 @@ and auxiliary coordinate roles for different data variables.
         fmt = 'NETCDF4'
         if cfa_options:
             g['cfa_options'] = cfa_options  
-    #--- End: if
     
     # ---------------------------------------------------------------
     # Flatten the sequence of intput fields
@@ -586,7 +605,6 @@ Create, if necessary, a netCDF dimension for string variables.
         # This string length dimension needs creating
         g['ncdim_to_size'][ncdim] = size
         g['netcdf'].createDimension(ncdim, size)
-    #--- End: if
 
     return ncdim
 #--- End: def
@@ -653,7 +671,6 @@ name.
     if ncdim not in g['CFA_ncdims']:
         g['CFA_ncdims'].add(ncdim)
         g['netcdf'].createDimension(ncdim, size)
-    #--- End: if
 
     return ncdim
 #--- End: def
@@ -959,11 +976,10 @@ int
     if isinstance(x, numpy_floating):
         return float(x)
 
-    raise TypeError("%r object is not JSON serializable: %r" %
-                    (type(x), x))
+    raise TypeError(
+        "{0!r} object is not JSON serializable: {1!r}".format(type(x), x))
 #--- End: def
 
-
 def _grid_ncdimensions(f, key, axis_to_ncdim, g=None):
     '''
 
@@ -1055,6 +1071,38 @@ def _write_dimension(ncdim, f, axis, axis_to_ncdim, is_unlimited=False, g=None):
         g['netcdf'].createDimension(ncdim, size)
 #--- End: def
 
+def _change_reference_datetime(coord, g=None):
+    '''
+
+:Parameters:
+
+    coord : cf.Coordinate
+
+    g : dict
+
+:Returns:
+
+    out : cf.Coordinate
+
+'''       
+    if not coord.Units.isreftime:
+        return coord
+
+    reference_datetime = g['reference_datetime']
+    if not reference_datetime:
+        return coord
+
+    coord2 = coord.copy()
+    try:
+        coord2.reference_datetime = reference_datetime
+    except ValueError:
+        raise ValueError(
+"Can't override coordinate reference date-time {0!r} with {1!r}".format(
+    coord.reference_datetime, reference_datetime))
+    else:
+        return coord2
+#--- End: def
+
 def _write_dimension_coordinate(f, axis, coord, key_to_ncvar, axis_to_ncdim,
                                 g=None):
     '''
@@ -1073,7 +1121,7 @@ a new netCDF bounds dimension.
    
     axis : str
 
-    coord : cf.Coordinate
+    coord : cf.DimensionCoordinate
 
     key_to_ncvar : dict
         Mapping of field item identifiers to netCDF dimension names.
@@ -1089,7 +1137,9 @@ a new netCDF bounds dimension.
         The netCDF name of the dimension coordinate.
 
 '''       
-    seen  = g['seen']
+    seen = g['seen']
+    
+    coord = _change_reference_datetime(coord, g)
 
     create = False
     if not _seen(coord, g=g):
@@ -1123,7 +1173,6 @@ a new netCDF bounds dimension.
                                 extra=extra, g=g)
     else:
         ncdim = seen[id(coord)]['ncvar']
-    #--- End: if
 
     key_to_ncvar[axis] = ncdim
 
@@ -1251,7 +1300,6 @@ variable name.
         if ncdim not in ncdim_to_size:
             ncdim_to_size[ncdim] = size
             g['netcdf'].createDimension(ncdim, size) #ncdim_to_size[ncdim])
-        #--- End: if
         
         ncvar = getattr(bounds, 'ncvar', '%s_bounds' % coord_ncvar)
         
@@ -1316,7 +1364,9 @@ then the input coordinate is not written.
     coordinates : list
         The updated list of netCDF auxiliary coordinate names.
 
-'''
+'''    
+    coord = _change_reference_datetime(coord, g)
+
     coord = coord.squeeze()
 
     if not _seen(coord, (), g=g):
@@ -1334,7 +1384,6 @@ then the input coordinate is not written.
         # This scalar coordinate has already been written to the
         # file
         ncvar = g['seen'][id(coord)]['ncvar']
-    #--- End: if
 
     axis_to_ncscalar[axis] = ncvar
 
@@ -1385,6 +1434,8 @@ then the input coordinate is not written.
 >>> coordinates = _write_auxiliary_coordinate(f, 'aux2', coordinates, g=g)
 
 '''
+    coord = _change_reference_datetime(coord, g)
+
     ncdimensions = _grid_ncdimensions(f, key, axis_to_ncdim, g=g)
 
     if _seen(coord, ncdimensions, g=g):
@@ -1456,7 +1507,7 @@ input coordinate is not written.
         
         if not hasattr(cm, 'measure'):
             raise ValueError(
-                "Can't write a cell measure without a 'measure' attribute")
+                "Can't create a cell measure variable without a 'measure' attribute")
 
         _create_netcdf_variable(ncvar, ncdimensions, cm, g=g)
     #--- End: if
@@ -1593,7 +1644,7 @@ created. The ``seen`` dictionary is updated for *cfvar*.
         The coordinate, cell measure or field object to write to the
         file.
 
-    omit : sequence, optional
+    omit : sequence of str, optional
 
     extra : dict, optional
 
@@ -1656,7 +1707,6 @@ created. The ``seen`` dictionary is updated for *cfvar*.
 
                 pda_args['axes'] = data._axes
                 pda_args['keep_in_memory'] = True
-        #--- End: if
     #--- End: if
 
     # Find the fill value (note that this is set in the call to
@@ -1705,7 +1755,7 @@ created. The ``seen`` dictionary is updated for *cfvar*.
                 **g['compression'])
         except (TypeError, RuntimeError):
             raise TypeError(
-"Can't write data type {0!r} (from {1!r}) to a {2} file. Use a netCDF4 format or use the single precision option.".format(
+"Can't write data type {0!r} (from {1!r}) to a {2} file. Use a netCDF4 format or use the 'single' or `datatype' parameters or change the datatype before writing".format(
     datatype, cfvar, g['netcdf'].file_format))
 
         _write_attributes(g['nc'][ncvar], netcdf_attrs)
@@ -1731,7 +1781,6 @@ created. The ``seen`` dictionary is updated for *cfvar*.
                 missing_value = getattr(cfvar, 'missing_value', None)            
                 missing_data = [value for value in (_FillValue, missing_value)
                                 if value is not None]
-            #--- End: if
 
             pda_args['revert_to_file'] = True
             
@@ -1785,10 +1834,7 @@ created. The ``seen`` dictionary is updated for *cfvar*.
         # Write a CFA variable 
         #---------------------------------------------------------
         _write_cfa_variable(ncvar, ncdimensions, netcdf_attrs, data, g=g)
-#                            fill_value=fill_value, g=g)
-
         return
-   #--- End: if
 #--- End: def
  
 def _write_a_field(f, add_to_seen=False, allow_data_expand_dims=True,
@@ -1876,7 +1922,6 @@ def _write_a_field(f, add_to_seen=False, allow_data_expand_dims=True,
                                                            key_to_ncvar,
                                                            axis_to_ncscalar,
                                                            g=g)
-            #--- End: if
         else:
             # --------------------------------------------------------
             # There is no dimension coordinate for this axis
@@ -1887,7 +1932,6 @@ def _write_a_field(f, add_to_seen=False, allow_data_expand_dims=True,
                 # the data array to include it.
                 f.expand_dims(0, axes=axis, i=True)
                 data_axes.append(axis)
-            #--- End: if
 
             # If the data array (now) spans this axis then create a
             # netCDF dimension for it
@@ -1900,7 +1944,6 @@ def _write_a_field(f, add_to_seen=False, allow_data_expand_dims=True,
                 ncdim, g['dimN'] = _check_name(ncdim, g['dimN'], g=g)
 
                 _write_dimension(ncdim, f, axis, axis_to_ncdim, g=g)
-             #--- End: if
         #--- End: if
 
     #--- End: for
@@ -1915,7 +1958,6 @@ def _write_a_field(f, add_to_seen=False, allow_data_expand_dims=True,
         coordinates = _write_auxiliary_coordinate(f, key, aux_coord,
                                                   coordinates, key_to_ncvar,
                                                   axis_to_ncdim, g=g)
-    #--- End: for
 
     # ----------------------------------------------------------------
     # Create cell measures variables
@@ -2011,8 +2053,6 @@ def _write_a_field(f, add_to_seen=False, allow_data_expand_dims=True,
         else:
             raise ValueError("Ooops: unknown coordinate reference type: %s" %
                              coordref_type)
-        #--- End: if
-
     #--- End: for
 
     # ----------------------------------------------------------------
@@ -2026,7 +2066,6 @@ def _write_a_field(f, add_to_seen=False, allow_data_expand_dims=True,
             _write_a_field(av, add_to_seen=True, allow_data_expand_dims=False, 
                            remove_crap_axes=1, g=g)
             ancillary_variables.append(seen[id(av)]['ncvar'])
-    #--- End: if
 
     # ----------------------------------------------------------------
     # Create the data variable
@@ -2079,12 +2118,11 @@ def _write_a_field(f, add_to_seen=False, allow_data_expand_dims=True,
             axis_to_ncdim, axis_to_ncscalar)
 
         extra['cell_methods'] = str(cell_methods)
-     #--- End: if
 
     # Create a new data variable
     _create_netcdf_variable(ncvar, ncdimensions, f,
-                            omit  = g['global_properties'],
-                            extra = extra,
+                            omit=g['global_properties'],
+                            extra=extra,
                             data_variable=True,
                             g=g)
     
@@ -2144,9 +2182,10 @@ write them to the netCDF4.Dataset.
                              ))
 
     # Put all non-standard CF properties (i.e. those not in the
-    # data_properties set) into the global_properties set.
+    # data_properties set) into the global_properties set, but
+    # omitting those which have been requested to be on variables.
     for f in fields:
-        for attr in set(f._simple_properties()) - global_properties:
+        for attr in set(f._simple_properties()) - global_properties - g['variable_attributes']:
             if attr not in data_properties:
                 global_properties.add(attr)
     #--- End: for
@@ -2167,7 +2206,7 @@ write them to the netCDF4.Dataset.
                     not equals(f.getprop(prop), prop0, traceback=False)):
                     global_properties.remove(prop)
                     break
-    #--- End: if
+    #--- End: for
 
     # Write the global properties to the file
     Conventions = __Conventions__
