@@ -2,9 +2,10 @@ from .netcdf.write import write as netcdf_write
 
 def write(fields, filename, fmt='NETCDF3_CLASSIC', overwrite=True,
           verbose=False, cfa_options=None, mode='w',
-          least_significant_digit=None, endian='native',
-          compress=0, fletcher32=False, no_shuffle=False,
-          datatype=None, single=False, double=False):
+          least_significant_digit=None, endian='native', compress=0,
+          fletcher32=False, no_shuffle=False, datatype=None,
+          single=False, double=False, reference_datetime=None,
+          variable_attributes=None):
     '''Write fields to a CF-netCDF or CFA-netCDF file.
     
 NetCDF dimension and variable names will be taken from variables'
@@ -25,7 +26,7 @@ and auxiliary coordinate roles for different data variables.
 
 :Parameters:
 
-    fields : (arbitrarily nested sequence of) Field or FieldList
+    fields : (arbitrarily nested sequence of) cf.Field or cf.FieldList
         The field or fields to write to the file.
 
     filename : str
@@ -214,5 +215,7 @@ and auxiliary coordinate roles for different data variables.
                      least_significant_digit=least_significant_digit,
                      endian=endian, compress=compress,
                      no_shuffle=no_shuffle, fletcher32=fletcher32,
-                     datatype=datatype, single=single, double=double)
+                     datatype=datatype, single=single, double=double,
+                     reference_datetime=reference_datetime,
+                     variable_attributes=variable_attributes)
 #--- End: def
